from typing import Any, Dict, Optional

from github.GithubObject import NonCompletableGithubObject

class NotificationSubject(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def latest_comment_url(self) -> Optional[str]: ...
    @property
    def title(self) -> str: ...
    @property
    def type(self) -> str: ...
    @property
    def url(self) -> Optional[str]: ...
