from datetime import datetime
from typing import Any, Dict, Optional

from github.GithubObject import CompletableGithubObject

class Download(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def accesskeyid(self) -> Optional[str]: ...
    @property
    def acl(self) -> Optional[str]: ...
    @property
    def bucket(self) -> Optional[str]: ...
    @property
    def content_type(self) -> Optional[str]: ...
    @property
    def created_at(self) -> Optional[datetime]: ...
    def delete(self) -> None: ...
    @property
    def description(self) -> Optional[str]: ...
    @property
    def download_count(self) -> Optional[int]: ...
    @property
    def expirationdate(self) -> Optional[datetime]: ...
    @property
    def html_url(self) -> Optional[str]: ...
    @property
    def id(self) -> int: ...
    @property
    def mime_type(self) -> Optional[str]: ...
    @property
    def name(self) -> Optional[str]: ...
    @property
    def path(self) -> Optional[str]: ...
    @property
    def policy(self) -> Optional[str]: ...
    @property
    def prefix(self) -> Optional[str]: ...
    @property
    def redirect(self) -> Optional[bool]: ...
    @property
    def s3_url(self) -> Optional[str]: ...
    @property
    def signature(self) -> Optional[str]: ...
    @property
    def size(self) -> Optional[int]: ...
    @property
    def url(self) -> Optional[str]: ...
