from datetime import datetime
from typing import Any, Dict

from github.GithubObject import NonCompletableGithubObject
from github.NamedUser import NamedUser
from github.Organization import Organization
from github.Repository import Repository

class Event(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def actor(self) -> NamedUser: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def id(self) -> str: ...
    @property
    def org(self) -> Organization: ...
    @property
    def payload(self) -> Dict[str, Any]: ...
    @property
    def public(self) -> bool: ...
    @property
    def repo(self) -> Repository: ...
    @property
    def type(self) -> str: ...
