from typing import Any, Dict, Optional

from github.GithubObject import NonCompletableGithubObject
from github.NamedUser import NamedUser
from github.Repository import Repository

class PullRequestPart(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def label(self) -> str: ...
    @property
    def ref(self) -> str: ...
    @property
    def repo(self) -> Repository: ...
    @property
    def sha(self) -> str: ...
    @property
    def user(self) -> Optional[NamedUser]: ...
