from typing import Any, Dict

from github.Commit import Commit
from github.GithubObject import NonCompletableGithubObject

class Tag(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def commit(self) -> Commit: ...
    @property
    def name(self) -> str: ...
    @property
    def tarball_url(self) -> str: ...
    @property
    def zipball_url(self) -> str: ...
