from datetime import datetime
from typing import Any, Dict, List, Optional

from github.GithubObject import NonCompletableGithubObject
from github.NamedUser import NamedUser

class StatsContributor(NonCompletableGithubObject):
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def author(self) -> NamedUser: ...
    @property
    def total(self) -> int: ...
    @property
    def weeks(self) -> List[Week]: ...
    class Week(NonCompletableGithubObject):
        def _initAttributes(self) -> None: ...
        def _useAttributes(self, attributes: Dict[str, int]) -> None: ...
        @property
        def a(self) -> int: ...
        @property
        def c(self) -> int: ...
        @property
        def d(self) -> int: ...
        @property
        def w(self) -> datetime: ...
