from typing import Any, Dict, List, Optional

from github.GithubObject import CompletableGithubObject
from github.NamedUser import NamedUser
from github.Team import Team

class RequiredPullRequestReviews(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def dismiss_stale_reviews(self) -> bool: ...
    @property
    def dismissal_teams(self) -> Optional[List[Team]]: ...
    @property
    def dismissal_users(self) -> Optional[List[NamedUser]]: ...
    @property
    def require_code_owner_reviews(self) -> bool: ...
    @property
    def required_approving_review_count(self) -> int: ...
    @property
    def url(self) -> int: ...
