from typing import Any, Dict

from github.GithubObject import CompletableGithubObject
from github.NamedUser import NamedUser
from github.Repository import Repository

class Invitation(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def created_at(self) -> str: ...
    @property
    def html_url(self) -> str: ...
    @property
    def id(self) -> int: ...
    @property
    def invitee(self) -> NamedUser: ...
    @property
    def inviter(self) -> NamedUser: ...
    @property
    def permissions(self) -> str: ...
    @property
    def repository(self) -> Repository: ...
    @property
    def url(self) -> str: ...
