from datetime import datetime
from typing import Dict

from github.GithubObject import NonCompletableGithubObject

class GitAuthor(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, str]) -> None: ...
    @property
    def date(self) -> datetime: ...
    @property
    def email(self) -> str: ...
    @property
    def name(self) -> str: ...
