from typing import Any, Dict

from github.GithubObject import NonCompletableGithubObject

class IssuePullRequest(NonCompletableGithubObject):
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def diff_url(self) -> str: ...
    @property
    def html_url(self) -> str: ...
    @property
    def patch_url(self) -> str: ...
