from typing import Any, Dict, Union

from github.GitAuthor import GitAuthor
from github.GithubObject import CompletableGithubObject
from github.GitObject import GitObject

class GitTag(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def message(self) -> str: ...
    @property
    def object(self) -> GitObject: ...
    @property
    def sha(self) -> str: ...
    @property
    def tag(self) -> str: ...
    @property
    def tagger(self) -> GitAuthor: ...
    @property
    def url(self) -> str: ...
