from typing import Any, Dict, List

from github.GithubObject import NonCompletableGithubObject

class HookDescription(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def events(self) -> List[str]: ...
    @property
    def name(self) -> str: ...
    @property
    def schema(self) -> List[List[str]]: ...
    @property
    def supported_events(self) -> List[str]: ...
