from typing import Any, Dict

from github.GithubObject import CompletableGithubObject
from github.NamedUser import NamedUser
from github.Organization import Organization

class Membership(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def url(self) -> str: ...
    @property
    def role(self) -> str: ...
    @property
    def organization_url(self) -> str: ...
    @property
    def organization(self) -> Organization: ...
    @property
    def user(self) -> NamedUser: ...
