from typing import Any, Dict, List

from github.GithubObject import CompletableGithubObject
from github.GitTreeElement import GitTreeElement

class GitTree(CompletableGithubObject):
    def __repr__(self) -> str: ...
    @property
    def _identity(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def sha(self) -> str: ...
    @property
    def tree(self) -> List[GitTreeElement]: ...
    @property
    def url(self) -> str: ...
