from typing import Any, Dict, List

from github.GitAuthor import GitAuthor
from github.GithubObject import CompletableGithubObject
from github.GitTree import GitTree

class GitCommit(CompletableGithubObject):
    def __repr__(self) -> str: ...
    @property
    def _identity(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def author(self) -> GitAuthor: ...
    @property
    def committer(self) -> GitAuthor: ...
    @property
    def html_url(self) -> str: ...
    @property
    def message(self) -> str: ...
    @property
    def parents(self) -> List[GitCommit]: ...
    @property
    def sha(self) -> str: ...
    @property
    def tree(self) -> GitTree: ...
    @property
    def url(self) -> str: ...
