from datetime import datetime
from typing import Any, Dict

from github.GithubObject import NonCompletableGithubObject

class Clones(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def count(self) -> int: ...
    @property
    def timestamp(self) -> datetime: ...
    @property
    def uniques(self) -> int: ...
