from typing import Any, Dict

from github.GithubObject import NonCompletableGithubObject
from github.Rate import Rate

class RateLimit(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def core(self) -> Rate: ...
    @property
    def graphql(self) -> Rate: ...
    @property
    def rate(self) -> Rate: ...
    @property
    def search(self) -> Rate: ...
