from typing import Dict

from github.GithubObject import NonCompletableGithubObject

class CommitStats(NonCompletableGithubObject):
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, int]) -> None: ...
    @property
    def additions(self) -> int: ...
    @property
    def deletions(self) -> int: ...
    @property
    def total(self) -> int: ...
