from typing import Any, Dict, Optional

from github.GithubObject import NonCompletableGithubObject

class GistFile(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def content(self) -> str: ...
    @property
    def filename(self) -> str: ...
    @property
    def language(self) -> Optional[str]: ...
    @property
    def raw_url(self) -> str: ...
    @property
    def size(self) -> int: ...
    @property
    def type(self) -> str: ...
