from datetime import datetime
from typing import Any, Dict

from github.GithubObject import CompletableGithubObject
from github.NamedUser import NamedUser

class TeamDiscussion(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def author(self) -> NamedUser: ...
    @property
    def body(self) -> str: ...
    @property
    def body_html(self) -> str: ...
    @property
    def body_version(self) -> str: ...
    @property
    def comments_count(self) -> int: ...
    @property
    def comments_url(self) -> str: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def html_url(self) -> str: ...
    @property
    def last_edited_at(self) -> datetime: ...
    @property
    def node_id(self) -> str: ...
    @property
    def number(self) -> int: ...
    @property
    def pinned(self) -> bool: ...
    @property
    def private(self) -> bool: ...
    @property
    def team_url(self) -> str: ...
    @property
    def title(self) -> str: ...
    @property
    def updated_at(self) -> datetime: ...
    @property
    def url(self) -> str: ...
