from datetime import datetime
from typing import Any, Dict

from github.GithubObject import NonCompletableGithubObject
from github.NamedUser import NamedUser
from github.TimelineEventSource import TimelineEventSource

class TimelineEvent(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def actor(self) -> NamedUser: ...
    @property
    def commit_id(self) -> str: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def event(self) -> str: ...
    @property
    def id(self) -> int: ...
    @property
    def node_id(self) -> str: ...
    @property
    def commit_url(self) -> str: ...
    @property
    def source(self) -> Any[TimelineEventSource, None]: ...
    @property
    def body(self) -> Any[str, None]: ...
    @property
    def author_association(self) -> Any[str, None]: ...
    @property
    def url(self) -> str: ...
