from typing import Any, Dict

from github.GithubObject import NonCompletableGithubObject, _ValuedAttribute
from github.PaginatedList import PaginatedList
from github.Repository import Repository

class Installation(NonCompletableGithubObject):
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def id(self) -> _ValuedAttribute: ...
    def get_repos(self) -> PaginatedList[Repository]: ...
